@@ -15,6 +15,7 @@ import dask
 from tornado import gen
 import toolz
 
+from .utils import without_property_validation, BOKEH_VERSION
 from ..diagnostics.progress_stream import nbytes_bar
 from .. import profile
 from ..utils import log_errors, parse_timedelta
@@ -69,6 +70,7 @@ class TaskStream(DashboardComponent):
         # Required for update callback
         self.task_stream_index = [0]
 
+    @without_property_validation
     def update(self, messages):
         with log_errors():
             index = messages['task-events']['index']
@@ -210,6 +212,7 @@ class MemoryUsage(DashboardComponent):
         )
         self.root.add_tools(hover)
 
+    @without_property_validation
     def update(self, messages):
         with log_errors():
             msg = messages['progress']
@@ -259,6 +262,7 @@ class Processing(DashboardComponent):
 
         self.root = fig
 
+    @without_property_validation
     def update(self, messages):
         with log_errors():
             msg = messages['processing']
@@ -309,10 +313,15 @@ class ProfilePlot(DashboardComponent):
         self.states = data.pop('states')
         self.source = ColumnDataSource(data=data)
 
+        @without_property_validation
         def cb(attr, old, new):
             with log_errors():
                 try:
-                    ind = new['1d']['indices'][0]
+                    selected = new.indices
+                except AttributeError:
+                    selected = new['1d']['indices']
+                try:
+                    ind = selected[0]
                 except IndexError:
                     return
                 data = profile.plot_data(self.states[ind], profile_interval)
@@ -321,7 +330,10 @@ class ProfilePlot(DashboardComponent):
                 self.source.data.update(data)
                 self.source.selected = old
 
-        self.source.on_change('selected', cb)
+        if BOKEH_VERSION >= '1.0.0':
+            self.source.selected.on_change('indices', cb)
+        else:
+            self.source.on_change('selected', cb)
 
         self.root = figure(tools='tap', **kwargs)
         self.root.quad('left', 'right', 'top', 'bottom', color='color',
@@ -362,6 +374,7 @@ class ProfilePlot(DashboardComponent):
         self.root.yaxis.visible = False
         self.root.grid.visible = False
 
+    @without_property_validation
     def update(self, state):
         with log_errors():
             self.state = state
@@ -403,12 +416,17 @@ class ProfileTimePlot(DashboardComponent):
 
         changing = [False]  # avoid repeated changes from within callback
 
+        @without_property_validation
         def cb(attr, old, new):
             if changing[0]:
                 return
             with log_errors():
+                if isinstance(new, list):  # bokeh >= 1.0
+                    selected = new
+                else:
+                    selected = new['1d']['indices']
                 try:
-                    ind = new['1d']['indices'][0]
+                    ind = selected[0]
                 except IndexError:
                     return
                 data = profile.plot_data(self.states[ind], profile_interval)
@@ -416,10 +434,16 @@ class ProfileTimePlot(DashboardComponent):
                 self.states.extend(data.pop('states'))
                 changing[0] = True  # don't recursively trigger callback
                 self.source.data.update(data)
-                self.source.selected = old
+                if isinstance(new, list):  # bokeh >= 1.0
+                    self.source.selected.indices = old
+                else:
+                    self.source.selected = old
                 changing[0] = False
 
-        self.source.on_change('selected', cb)
+        if BOKEH_VERSION >= '1.0.0':
+            self.source.selected.on_change('indices', cb)
+        else:
+            self.source.on_change('selected', cb)
 
         self.profile_plot = figure(tools='tap', height=400, **kwargs)
         r = self.profile_plot.quad('left', 'right', 'top', 'bottom', color='color',
@@ -488,7 +512,10 @@ class ProfileTimePlot(DashboardComponent):
                     self.start = self.stop = None
                 self.trigger_update(update_metadata=False)
 
-        self.ts_source.on_change('selected', ts_change)
+        if BOKEH_VERSION >= '1.0.0':
+            self.ts_source.selected.on_change('indices', ts_change)
+        else:
+            self.ts_source.on_change('selected', ts_change)
 
         self.reset_button = Button(label="Reset", button_type="success")
         self.reset_button.on_click(lambda: self.update(self.state) )
@@ -510,6 +537,7 @@ class ProfileTimePlot(DashboardComponent):
                                self.update_button, sizing_mode='scale_width'),
                            self.profile_plot, self.ts_plot, **kwargs)
 
+    @without_property_validation
     def update(self, state, metadata=None):
         with log_errors():
             self.state = state
@@ -529,6 +557,7 @@ class ProfileTimePlot(DashboardComponent):
 
                 self.ts_source.data.update(self.ts)
 
+    @without_property_validation
     def trigger_update(self, update_metadata=True):
         @gen.coroutine
         def cb():
@@ -566,12 +595,17 @@ class ProfileServer(DashboardComponent):
 
         changing = [False]  # avoid repeated changes from within callback
 
+        @without_property_validation
         def cb(attr, old, new):
             if changing[0]:
                 return
             with log_errors():
+                if isinstance(new, list):  # bokeh >= 1.0
+                    selected = new
+                else:
+                    selected = new['1d']['indices']
                 try:
-                    ind = new['1d']['indices'][0]
+                    ind = selected[0]
                 except IndexError:
                     return
                 data = profile.plot_data(self.states[ind], profile_interval)
@@ -579,10 +613,16 @@ class ProfileServer(DashboardComponent):
                 self.states.extend(data.pop('states'))
                 changing[0] = True  # don't recursively trigger callback
                 self.source.data.update(data)
-                self.source.selected = old
+                if isinstance(new, list):  # bokeh >= 1.0
+                    self.source.selected.indices = old
+                else:
+                    self.source.selected = old
                 changing[0] = False
 
-        self.source.on_change('selected', cb)
+        if BOKEH_VERSION >= '1.0.0':
+            self.source.selected.on_change('indices', cb)
+        else:
+            self.source.on_change('selected', cb)
 
         self.profile_plot = figure(tools='tap', height=400, **kwargs)
         r = self.profile_plot.quad('left', 'right', 'top', 'bottom', color='color',
@@ -651,7 +691,10 @@ class ProfileServer(DashboardComponent):
                     self.start = self.stop = None
                 self.trigger_update()
 
-        self.ts_source.on_change('selected', ts_change)
+        if BOKEH_VERSION >= '1.0.0':
+            self.ts_source.selected.on_change('indices', ts_change)
+        else:
+            self.ts_source.on_change('selected', ts_change)
 
         self.reset_button = Button(label="Reset", button_type="success")
         self.reset_button.on_click(lambda: self.update(self.state))
@@ -663,6 +706,7 @@ class ProfileServer(DashboardComponent):
                                sizing_mode='scale_width'),
                            self.profile_plot, self.ts_plot, **kwargs)
 
+    @without_property_validation
     def update(self, state):
         with log_errors():
             self.state = state
@@ -670,6 +714,7 @@ class ProfileServer(DashboardComponent):
             self.states = data.pop('states')
             self.source.data.update(data)
 
+    @without_property_validation
     def trigger_update(self):
         self.state = profile.get_profile(self.log, start=self.start, stop=self.stop)
         data = profile.plot_data(self.state, profile_interval)
