@@ -6,6 +6,7 @@ from datetime import timedelta
 import json
 from operator import add, mul
 import sys
+from time import sleep
 
 import dask
 from dask import delayed
@@ -193,6 +194,43 @@ def test_remove_worker_from_scheduler(s, a, b):
     s.validate_state()
 
 
+@gen_cluster(config={"distributed.scheduler.events-cleanup-delay": "10 ms"})
+def test_clear_events_worker_removal(s, a, b):
+    assert a.address in s.events
+    assert a.address in s.ncores
+    assert b.address in s.events
+    assert b.address in s.ncores
+
+    s.remove_worker(address=a.address)
+    # Shortly after removal, the events should still be there
+    assert a.address in s.events
+    assert a.address not in s.ncores
+    s.validate_state()
+
+    start = time()
+    while a.address in s.events:
+        yield gen.sleep(0.01)
+        assert time() < start + 2
+    assert b.address in s.events
+
+
+@gen_cluster(config={"distributed.scheduler.events-cleanup-delay": "10 ms"}, client=True)
+def test_clear_events_client_removal(c, s, a, b):
+    assert c.id in s.events
+    s.remove_client(c.id)
+
+    assert c.id in s.events
+    assert c.id not in s.clients
+    assert c not in s.clients
+
+    s.remove_client(c.id)
+    # If it doesn't reconnect after a given time, the events log should be cleared
+    start = time()
+    while c.id in s.events:
+        yield gen.sleep(0.01)
+        assert time() < start + 2
+
+
 @gen_cluster()
 def test_add_worker(s, a, b):
     w = Worker(s.ip, s.port, ncores=3)
@@ -441,7 +479,7 @@ def test_worker_name():
     s.start(0)
     w = Worker(s.ip, s.port, name='alice')
     yield w._start()
-    assert s.worker_info[w.address]['name'] == 'alice'
+    assert s.workers[w.address].name == 'alice'
     assert s.aliases['alice'] == w.address
 
     with pytest.raises(ValueError):
@@ -574,7 +612,7 @@ def test_scheduler_sees_memory_limits(s):
     w = Worker(s.ip, s.port, ncores=3, memory_limit=12345)
     yield w._start(0)
 
-    assert s.worker_info[w.address]['memory_limit'] == 12345
+    assert s.workers[w.address].memory_limit == 12345
     yield w._close()
 
 
@@ -731,26 +769,28 @@ def test_file_descriptors(c, s):
         assert time() < start + 3
 
 
+@slow
 @nodebug
 @gen_cluster(client=True)
 def test_learn_occupancy(c, s, a, b):
-    futures = c.map(slowinc, range(1000), delay=0.01)
+    futures = c.map(slowinc, range(1000), delay=0.2)
     while sum(len(ts.who_has) for ts in s.tasks.values()) < 10:
         yield gen.sleep(0.01)
 
-    assert 1 < s.total_occupancy < 40
+    assert 100 < s.total_occupancy < 1000
     for w in [a, b]:
-        assert 1 < s.workers[w.address].occupancy < 20
+        assert 50 < s.workers[w.address].occupancy < 700
 
 
+@slow
 @nodebug
 @gen_cluster(client=True)
 def test_learn_occupancy_2(c, s, a, b):
-    future = c.map(slowinc, range(1000), delay=0.1)
+    future = c.map(slowinc, range(1000), delay=0.2)
     while not any(ts.who_has for ts in s.tasks.values()):
         yield gen.sleep(0.01)
 
-    assert 50 < s.total_occupancy < 200
+    assert 100 < s.total_occupancy < 1000
 
 
 @gen_cluster(client=True)
@@ -1300,6 +1340,21 @@ def test_dont_recompute_if_persisted_4(c, s, a, b):
     assert len(new) > len(old)
 
 
+@gen_cluster(client=True)
+def test_dont_forget_released_keys(c, s, a, b):
+    x = c.submit(inc, 1, key='x')
+    y = c.submit(inc, x, key='y')
+    z = c.submit(dec, x, key='z')
+    del x
+    yield wait([y, z])
+    del z
+
+    while 'z' in s.tasks:
+        yield gen.sleep(0.01)
+
+    assert 'x' in s.tasks
+
+
 @gen_cluster(client=True)
 def test_dont_recompute_if_erred(c, s, a, b):
     x = delayed(inc)(1, dask_key_name='x')
@@ -1325,3 +1380,44 @@ def test_closing_scheduler_closes_workers(s, a, b):
     while a.status != 'closed' or b.status != 'closed':
         yield gen.sleep(0.01)
         assert time() < start + 2
+
+
+@gen_cluster(client=True, ncores=[('127.0.0.1', 1)],
+             worker_kwargs={'resources': {'A': 1}})
+def test_resources_reset_after_cancelled_task(c, s, w):
+    future = c.submit(sleep, 0.2, resources={'A': 1})
+
+    while not w.executing:
+        yield gen.sleep(0.01)
+
+    yield future.cancel()
+
+    while w.executing:
+        yield gen.sleep(0.01)
+
+    assert not s.workers[w.address].used_resources['A']
+    assert w.available_resources == {'A': 1}
+
+    yield c.submit(inc, 1, resources={'A': 1})
+
+
+@gen_cluster(client=True)
+def test_gh2187(c, s, a, b):
+    def foo():
+        return 'foo'
+
+    def bar(x):
+        return x + 'bar'
+
+    def baz(x):
+        sleep(0.1)
+        return x + 'baz'
+
+    x = c.submit(foo, key='x')
+    y = c.submit(bar, x, key='y')
+    yield y
+    z = c.submit(baz, y, key='z')
+    del y
+    yield gen.sleep(0.1)
+    f = c.submit(bar, x, key='y')
+    yield f
